unit U_TreblePuzzle;
{Copyright  2008, Gary Darby,  www.DelphiForFun.org
 This program may be used or modified for any non-commercial purpose
 so long as this original notice remains in place.
 All other rights are reserved
 }

{Puzzle 1:  Find all 5 digit numbers that contain the same
5 digits when they are multiplied by 3.
A Version 2 button provides a more generalized solution for other
number sizes anbd multipliers}

{Puzzle 2:

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ShellAPI, ComCtrls;

type
  TForm1 = class(TForm)
    StaticText2: TStaticText;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    SolutionLbl: TLabel;
    Memo1: TMemo;
    Puzzle1V1Btn: TButton;
    ListBox1: TListBox;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    SpinEdit1: TSpinEdit;
    SpinEdit2: TSpinEdit;
    Puzzle1V2Btn: TButton;
    TabSheet2: TTabSheet;
    Memo2: TMemo;
    Puzzle2Btn: TButton;
    ListBox2: TListBox;
    procedure Puzzle1V1BtnClick(Sender: TObject);
    procedure Puzzle1V2BtnClick(Sender: TObject);
    procedure Label2Click(Sender: TObject);
    procedure Puzzle2BtnClick(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{********** Swap ***********}
procedure swap(var a,b:integer);
{Swap 2 integers, "a" and "b"}
  var t:integer;
  begin  t:=a;  a:=b;  b:=t;  end;

{************* SortDigits *********8}
function sortdigits(const InputN:integer):integer;
{Sort the digits of an integer into ascending order}
var
  i,j,n,size:integer;
  digits:array[1..10] of integer;
begin
  n:=abs(inputN);  {Get a working version of the input number}
  size:=0;  {initialize the position counter}

  {1. Put the digits into an array, one digit per entry}
  while n>0 do
  begin
    inc(size); {index for the next digit working from right to left}
    digits[size]:=n mod 10;  {save the rightmost (low order) digit}
    n:=n div 10; {get rid of the rightmost digit in N}
  end;
  {2. Sort them smallest to largest high}
  for i:=1 to size-1 do {starting with each digit}
  for j:= i+1 to size do {Check from there to the end}
  {and swap it with any that are lower so by the time that j gets to size,
   the lowest of the remain unchecked digits is in position i}
  if digits[j]<digits[i] then swap(digits[i], digits[j]);

  {3. Rebuild the number as the result}
  result:=1;
  for i:=1 to size do result:=result*10+digits[i];
end;

{************* V1BtnClick *************}
procedure TForm1.Puzzle1V1BtnClick(Sender: TObject);
{User clicked the button doe the Version 1 method}
var
  i:integer;
begin
  listbox1.clear;
  for i:=10000 to 33333 do {test from smallest 5 digit nbr to the largest which
                             will still be 5 digits when multiplied by 3}
  begin
    {find numbers represented by sorted version of i and 3*i
     digits and compare them}
    if sortdigits(i)=sortdigits(3*i)
    then listbox1.items.add(format('%d x 3 = %d',[i,3*i]));
  end;
  solutionLbl.caption:=inttostr(listbox1.items.count)+' solutions found';
end;

{******** Intpower ******}
function intpower(x,n:integer):int64;
{For version 2 we'll need to calculate the range of numbers to check by
 10 to the Nth power.  This version calclates X to the Nth power (X^n) }
var i:integer;
begin
  result:= 1; {default result}
  for i:=1 to n-1 do result:=result*x;
end;

{********** V2btnClick ************}
procedure TForm1.Puzzle1V2BtnClick(Sender: TObject);
{A second version that lets the user choose the number size and
 the  multiplier}
var
  i:integer;
  start, stop:int64;
  N:integer; {the count of digits on the number}
  V:integer; {the multiplier}
begin
  listbox1.clear;
  N:=spinedit1.value;
  V:=spinedit2.value;
  start:=intpower(10,N); {"Start" is smallest N digit integer}
  {stop is the largest value that will still have N only digits}
  stop:=(10*start) div V;
  {Make sure that start and stop are both within the range of valid
   32 bit integers}
  if (start<high(integer)) and (stop<high(integer)) then
  for i:=start to stop do {Start & stop are valid, check all integers in the range}
  begin
    {find numbers represented by sorted versions of the
     digits in the number  and V times the number and compare them}
    if sortdigits(i)=sortdigits(V*i) {if equal, we found a solution}
    then listbox1.items.add(format('%d x %d = %d',[i,V, V*i]));
  end
  else beep;
  solutionLbl.caption:=inttostr(listbox1.items.count)+' solutions found';
end;

procedure TForm1.Label2Click(Sender: TObject);
begin
   ShellExecute(Handle, 'open', 'http://www.delphiforfun.org/',
                 nil, nil, SW_SHOWNORMAL) ;
end;

{************** Puzzle2BtnClick *************}
procedure TForm1.Puzzle2BtnClick(Sender: TObject);
{Solve the second puzzle, find 3 digit numbers which when combined with
 double and triple that number, contain all of the digits 1 through 9 exactly once.}

       {*********** Check **********}
       procedure check(const i:integer; var dcheck:string);
       {Examine digits of i and set positions of "dcheck" indexed by digit
        values to "Y" for the 1st occurrence of that digit}
       var
         n,d:integer;
        begin
          n:=i;
          while n>0 do {for all digits}
          begin
            d:=n mod 10 ;   {d = the next units digit}
            if (d>0) then
              if dcheck[d]='N'
              then dcheck[d]:='Y' {set position d to "Y" for 1st occurence of d}
              else dcheck[d]:='X';{after the 1st, set position to "X"}

            n:=n div 10; {drop the units digit}
          end;
        end;

var
  i,d:integer;
  n:integer;
  digitcheck:string;
begin
  listbox2.Clear;
  for i:=100 to 333 do  {check all 3 digit numbers}
  begin
    digitcheck:='NNNNNNNNN';   {9 "N"s}
    check(i, digitcheck); {check digits of i}
    check(2*i, digitcheck); {check digits of 2i}
    check(3*i, digitcheck); {check digits of 3i}
    if digitcheck='YYYYYYYYY' {9 "Y"s ==> one each of digits 1 through 9}
    then listbox2.items.add(format('n=%d, 2n=%d, 3n=%d',[i,2*i,3*i]));
  end;
end;

end.
